import os
import numpy as np
import open3d
from pyquaternion import Quaternion
from mujoco_py import load_model_from_xml, MjSim, MjViewer


MODEL_XML = """
<?xml version="1.0" ?>
<mujoco>
   <asset>
        <texture builtin="flat" name="object_tex" height="32" width="32" rgb1="1 1 1" type="cube"></texture>
        <material name="object_mat" shininess="0.03" specular="0.75" texture="object_tex"></material>
       <mesh file="knife.stl" name="mesh_name" scale="2 2 2"></mesh>
   </asset>

   <worldbody>

       <body name="object" pos="0 0 0">
            <geom type='mesh' pos="0 0 0" mesh="mesh_name" name="geom_name" material="object_mat"/>
       </body>

   </worldbody>
</mujoco>
"""


class ObjModel:

    def __init__(self):
        self.model = load_model_from_xml(MODEL_XML)
        self.sim = MjSim(self.model)
        self.viewer = MjViewer(self.sim)
        self.step = 0
        self.obj_bid = self.sim.model.body_name2id('object')

        curr_dir = os.path.dirname(os.path.abspath(__file__))
        ply_mesh = open3d.read_triangle_mesh('knife.ply'))
        stl_mesh = open3d.read_triangle_mesh(dir_path)

        self.body_xpos = self.sim.data.body_xpos[self.obj_bid].ravel()
        self.body_xquat = self.sim.data.body_xquat[self.obj_bid].ravel()
        self.geom_pos = self.sim.model.geom_pos.ravel()
        self.geom_quat = self.sim.model.geom_quat.ravel()

        print('Number of vertices: ', self.sim.model.mesh_vert.shape)
        print('Number of meshes: ', self.model.nmesh)
        print('Number of vertices in all meshes: ', self.model.nmeshvert)
        print('Number of vertices in each mesh: ', self.model.mesh_vertnum)
        print('Starting address of each mesh: ', self.model.mesh_vertadr)
        obj_mesh_idx = 0
        print('Number of vertices in mujoco mesh: ', self.model.mesh_vertnum[obj_mesh_idx])
        print('Number of vertices in ply mesh: ', np.asarray(ply_mesh.vertices).shape)
        print('Number of points in stl mesh: ', np.asarray(stl_mesh.vertices).shape)
        print('Number of unique points in stl mesh: ', np.unique(np.asarray(stl_mesh.vertices), axis=0).shape)

    def reset_model(self):
        self.model.body_pos[self.obj_bid, 0] = np.random.uniform(low=-0.15, high=0.15)
        self.model.body_pos[self.obj_bid, 1] = np.random.uniform(low=-0.15, high=0.3)
        angle = np.random.uniform(low=0, high=360)
        quat = Quaternion(axis=[0, 0, 1], degrees=angle).elements
        self.model.body_quat[self.obj_bid] = quat
        self.sim.forward()

    def debug(self):
        cnt = 0
        while True:
            cnt += 1
            if cnt % 500 == 0:
                self.reset_model()

            for ind in [51170, 52750, 51182, 51714, 52754]:
                vert = self.model.mesh_vert[ind]
                vert = Quaternion(self.geom_quat).rotate(vert) + self.geom_pos
                vert = Quaternion(self.body_xquat).rotate(vert) + self.body_xpos
                self.viewer.add_marker(pos=vert, rgba=[0.2, 0.8, 0.2, 1.0],
                                       size=[0.01, 0.01, 0.01],
                                       label="")
            self.viewer.render()


if __name__ == '__main__':
    objmodel = ObjModel()
    objmodel.debug()

